/**
 * The ViewPort component is the main interface expose to the user.
 * It contains the Ext.tree.TreePanel, on the left side and the
 * Ext.TabPanel on the right side.
 * The TreePanel lists {@link Ext.data.NodeInterface node}.
 * Actions are triggered by clicking on a node.
 *
 * Two actions are possible when a leaf node is clicked.
 * In the first case a new Tab is create in the
 * TabPanel displaying configuration parameters and the results of the operation.
 * In the second case a window pop-up containing information, wizard, ....
 *
 * The choice between the two action is determined by the xtype of the
 * component configuration embedded in the node.
 * In the first case the xtype is *panel* while in the second one is *window*.
 *
 */
Ext.define('App.container.Viewport', {

    extend: 'Ext.container.Viewport',
    alias: 'widget.xviewport',
    requires: [
        'App.container.plugin.Login',
        'Ext.layout.container.Border',
        'Ext.layout.container.Fit',
        'Ext.tab.Panel',
        'Ext.Template',
        'Ext.tree.Panel'
    ],
    uses: [
        'App.form.Panel',
        'App.grid.Panel',
        'App.panel.GridWithFilter',
        'App.panel.WithUrlSelector',
        'Ext.panel.Panel',
        'Ext.window.Window'
    ],
    /**
     * @property {String[]}
     * The list of {@link Ext.data.NodeInterface#text text} attribute
     * defining the path of the nodes to be opened at start up.
     */
    defaultTabPath: null,

    /**
     * @property {Boolean}
     * The property is true when the user pass the login procedure.
     */
    logged: false,

    /**
     * @property {String}
     * The template use to build the title of each tab.
     * The argument {0} is the name of the node while {1} is the
     * name of the leaf.
     */
    tabTitleTpl: "{0}: {1}",

    /**
     * @property {String} treePanelTitle
     * The title of the tree panel.
     * By default it is the name of the application.
     */
    treePanelTitle: undefined,

    // private short cuts
    tabPanel: null,
    treePanel: null,

    // private method requires by the Ext JS component model
    initComponent: function (config) {

        var treeStoreCfg,
            viewportCfg;

        // apply the user configuration if any
        Ext.apply(this, config);

        // the TreeStore configuration
        treeStoreCfg = {
            root: {
                expanded: true,
                children: App.config.treeNodes,
                text: App.name
            },
            xtype: 'tree'
        };

        // title of the tree panel
        if (!this.treePanelTitle) {
            this.treePanelTitle = App.name;
        }

        // predefined configuration of the view port
        viewportCfg = {
            layout: 'border',
            title: 'Ext Layout Browser',
            items: [{
                height: 25,
                hidden: true,
                itemId: 'topBar',
                region: 'north',
                xtype: 'toolbar'
            }, {
                // autoScroll is mandatory to avoid problem
                // when the first tab is deleted
                autoScroll: true,
                collapsible: true,
                itemId: 'treePanel',
                region: 'west',
                rootVisible: false,
                split: true,
                store: treeStoreCfg,
                title: this.treePanelTitle,
                width: 200,
                xtype: 'treepanel'
            }, {
                autoDestroy: false,
                autoScroll: true,
                bodyPadding: 1,
                defaults: {layout: 'fit'},
                enableTabScroll: true,
                itemId: 'tabPanel',
                region: 'center',
                tabPosition: 'top',
                xtype: 'tabpanel'
            }, {
                height: 25,
                hidden: true,
                itemId: 'bottomBar',
                region: 'south',
                xtype: 'toolbar'
            }]
        };

        // apply the predefined configuration
        Ext.apply(this, viewportCfg);

        // compile template
        this.tabTitleTpl = Ext.create('Ext.Template', this.tabTitleTpl);
        this.tabTitleTpl.compile();

        //initialise the base class
        this.callParent(arguments);

        // define short cuts
        this.tabPanel = this.getComponent('tabPanel');
        this.treePanel = this.getComponent('treePanel');

        // Add handler to create tab/window and their widget
        this.treePanel.on('itemclick', this.onItemClick, this);

        // disable ContextMenu everywhere in the ViewPort
        this.getEl().on('contextmenu', this.onContextMenu);

        // open the default Tab
        this.openDefaultTab();
    },

    // Private method requires by the Ext JS component model
    beforeDestroy: function () {

        this.treePanel.un('itemclick', this.onItemClick, this);
        this.getEl().un('contextmenu', this.onContextMenu);

        this.callParent(arguments);
    },

    /**
     * deactivate the context menu everywhere in the ViewPort.
     *
     * @param {Ext.EventObject} event
     *
     */
    onContextMenu: function (event) {

        "use strict";
        event.stopEvent();
    },

    /**
     * Expand or collapse node.
     * Create tab/window or activate an existing tab.
     *
     * @param {Ext.view.View} view
     * @param {Ext.data.Model} record
     * @param {HTMLElement} item
     * @param {Number} index
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onItemClick: function (view, record, item, index, event, eOpts) {

        "use strict";

        var cfg,
            li,
            parent,
            tabId,
            wdgcfg,
            wdgtype,
            window;

        // protection
        if (!record.isNode  || !item) {
            return;
        }

        // expand or collapse node
        if (!record.isLeaf()) {

            if (record.isExpanded()) {
                record.collapse();

            } else {
                record.expand();
            }

            return;
        }

        // Create or activate the tab for a leafs
        // configuration of the embedded widget
        wdgcfg = record.raw.cfg;
        wdgtype = wdgcfg.xtype;

        // standalone window
        if (wdgtype === 'window') {
            window = Ext.create('Ext.window.Window', wdgcfg);
            window.show();
            return;
        }

        // unique tab identifier based on the node path
        tabId = record.getPath('text', '_');

        // activate an existing tab
        if (this.tabPanel.getComponent(tabId)) {
            this.tabPanel.setActiveTab(tabId);
            return;
        }

        // tab configuration
        parent = record.parentNode;

        li = [parent.get("text"), record.get("text")];

        cfg = {
            closable: true,
            itemId: tabId,
            title: this.tabTitleTpl.apply(li)
        };

        // embed single panel in the tab panel
        // other widget as children
        if (wdgtype === 'panel') {
            delete wdgcfg.xtype;
            Ext.apply(cfg, wdgcfg);

        } else {
            cfg.items = [wdgcfg];
        }

        // create a new tab and activate it
        this.tabPanel.add(cfg);
        this.tabPanel.setActiveTab(tabId);
    },

    /**
     * Open the default tab defined in #defaultTabPath
     *
     */
    openDefaultTab: function () {

        "use strict";

        var i,
            node,
            path = this.defaultTabPath,
            treePanel = this.getComponent('treePanel');

        // protection against undefined path
        if (!path) {
            return;
        }

        // get the node of the path
        node = treePanel.getStore().getRootNode();
        for (i = 0; i < path.length; i += 1) {
            node = node.findChild('text', path[i]);
        }

        // create the tab for the last one
        this.onItemClick(treePanel.getView(), node);
    }
});