/**
 * The DirectStore is a store to read/write in the database using
 * the Ext.Direct and JSON protocols.
 *
 * It use the App.data.proxy.Direct to work with the DbSvc service running
 * on the server side.
 *
 * It also provide a set of method to filter the store passing by the server.
 *
 * **Note**: this class inherits of the config options from:
 *
 * - **{@link Ext.data.proxy.Direct proxy}**
 *
 *   - **{@link Ext.data.proxy.Direct#extraParams extraParams}**
 *   - **{@link Ext.data.proxy.Direct#pageParam pageParam}**
 *
 * - **{@link Ext.data.reader.Json JsonReader}**
 *
 *   - **{@link Ext.data.reader.Json#cfg-root root}**
 *   - **{@link Ext.data.reader.Json#idProperty idProperty}**
 *   - **{@link Ext.data.reader.Json#successProperty successProperty}**
 *   - **{@link Ext.data.reader.Json#totalProperty totalProperty}**

 */
Ext.define('App.data.DirectStore', {

    extend: 'Ext.data.Store',
    alias: 'widget.xdirectstore',
    requires: 'App.data.proxy.Direct',

    // private property.
    // List of the where condition at startup.
    // It is a copy of the baseParams.where which can be modified by filter.
    // The intialWhere configuration parameter can be used reseting filter
    //
    initialWhere: undefined,

    /**
     * @constructor
     */
    constructor: function (config) {

        var i,
            proxy;

        // the proxy
        if (!config.proxy) {
            proxy = {
                type: 'xdirect'
            };
            Ext.copyTo(proxy, config, ['extraParams',
                                       'idProperty',
                                       'pageParam',
                                       'root',
                                       'successProperty',
                                       'totalProperty']);
            config.proxy = proxy;
        }

        // copy of the initial where condition
        if (config.extraParams && config.extraParams.where) {
            this.initialWhere = [];
            for (i = 0; i < config.extraParams.where.length; i += 1) {
                this.initialWhere.push(config.extraParams.where[i]);
            }
        }

        // initialise the base class
        this.callParent(arguments);

        // update the total number of records
        this.on('write', this.onWrite);
    },

    // private method request by the Ext JS component model
    destroy: function () {

        this.un('write', this.onWrite);
        this.callParent(arguments);
    },

    /**
     * Handler to update the total number of records after a write action.
     *
     * @param {Ext.data.Store} store
     * @param {Ext.data.Operation} operation
     * Understood action are *create* and *destroy*
     * @param {Object} eOpts
     *
     */
    onWrite: function (store, operation, eOpts) {

        "use strict";

        switch (operation.action) {
        case 'create':
            store.totalLength += 1;
            break;

        case 'destroy':
            store.totalLength -= 1;
            break;
        }
    },

    /**
     * Restore the where conditions to the original values.
     * Useful to remove remote filter condition.
     */
    restoreWhere: function () {

        "use strict";

        var conditions = [],
            i;

        if (this.extraParams.where) {

            if (this.initialWhere) {

                for (i = 0; i < this.initialWhere.length; i += 1) {
                    conditions.push(this.initialWhere[i]);
                }
                this.getProxy().setExtraParam('where', conditions);

            } else {
                delete this.getProxy().extraParams.where;
            }
        }
    }
});