/**
 * Implement the server protocol.
 *
 * **Note**: this class inherits of the config options from the JsonReader:
 *
 * - **{@link Ext.data.reader.Json JsonReader}**
 *
 *   - **{@link Ext.data.reader.Json#cfg-root root}**
 *   - **{@link Ext.data.reader.Json#idProperty idProperty}**
 *   - **{@link Ext.data.reader.Json#successProperty successProperty}**
 *   - **{@link Ext.data.reader.Json#totalProperty totalProperty}**
 *
 * * **Note**: each operation contains at least 3 parameters:
 *
 *      - dbFields {Array[]}
 *        The definition of the database field ['tablename', 'fieldname']
 *
 *      - tableName {String}
 *        The name of the database table.
 *
 *      - records {Array}
 *        The content of the records depend of the operation.
 *        For create or update operation, the record is an Object containing
 *        the fields and their values. the field is encoded *TableField*.
 *        For a destroy operation it is just a list of *ids*.
 *
 * the first two parameters are defined in the
 * {@link Ext.data.proxy.Direct#extraParams extraParams} attribute.
 *
 * @since 0.6.0.0
 *
 */
Ext.define('App.data.proxy.Direct', {

    extend: 'Ext.data.proxy.Direct',
    alias: 'proxy.xdirect',

    /**
     * @constructor
     */
    constructor: function (config) {

        var reader;

        // the direct API tu run with the DbSvc service
        if (!config.api) {
            config.api = {
                create: Dbui.create,
                destroy: Dbui.destroy,
                read: Dbui.read,
                update: Dbui.update
            };
        }

        // the JSON reader
        if (!config.reader) {
            reader = {
                type: 'json'
            };
            Ext.copyTo(reader, config, 'root,idProperty,successProperty,totalProperty');
            config.reader = reader;
        }

        // the JSOn writer
        if (!config.writer) {
            config.writer = {
                type: 'json',
                allowSingle: false,
                writeAllFields: false
            };
        }

        // initialise the base class
        this.callParent(arguments);
    },

    /**
     * Implement the server protocol for the create, read, update and
     * destroy operation.
     *
     * **Note**: This method is a copy of the Ext.data.proxy.Direct.doRequest method
     * adapted to the create, update and destroy operations.
     *
     * @param {Ext.data.Operation} operation
     * @param {Object} callback
     * @param {Object} scope
     *
     */
    doRequest: function (operation, callback, scope) {

        "use strict";

        var me = this,
            writer = me.getWriter(),
            request = me.buildRequest(operation),
            params = request.params,
            args = [],
            i,
            fieldId = App.encodeField(operation.params.tableName, 'id'),
            fn,
            method,
            records;

        if (!me.methodsResolved) {
            me.resolveMethods();
        }

        fn = me.api[request.action] || me.directFn;

        if (operation.allowWrite()) {
            request = writer.write(request);
        }

        switch (operation.action) {

        case 'read':
            method = fn.directCfg.method;
            args = method.getArgs(params, me.paramOrder, me.paramsAsHash);
            break;

        case 'create':
            params.records = request.jsonData;
            args.push(params);
            break;

        case 'update':
            params.records = request.jsonData;

            // abandon the update if the proxy is not yet ready
            for (i = 0; i < params.records; i +=1) {
                if (!params.records[fieldId]) {
                    return;
                }
            }

            args.push(params);
            break;

        case 'destroy':

            // build the list of id to be destroy
            params.records = [];

            records = operation.getRecords();
            for (i = 0; i < records.length; i += 1) {
                params.records.push(records[i].get(fieldId));
            }

            // abandon the transaction if the proxy is not yet ready
            if (records === null || params.records.length === 0) {
                return;
            }

            args.push(params);
            break;
        }

        Ext.apply(request, {
            args: args,
            directFn: fn
        });

        args.push(me.createRequestCallback(request, operation, callback, scope), me);
        fn.apply(window, args);
    }
});
