/**
 * This class is used as a mixin.
 * The main method translate the Ext.grid.View content into a CSV string.
 *
 * @since 0.6.2
 *
 */
Ext.define('App.grid.mixin.ExportCSV', {

    extend: 'App.grid.mixin.ExportBase',
    config: {

        /**
         * @cfg {String}
         * The CSV delimiter for string.
         */
        delimiter: '"',

        /**
         * @cfg {String}
         * The CSV field separator.
         *
         */
        separator: ','

    },

    //
    // Surround the value with the CSV delimiter.
    // Delimiter are only applied to non-numeric value.
    //
    // @param {String/Number} value
    //
    //
    delimit: function (value) {

        "use strict";

        if (!Ext.isNumeric(value)) {
            value = this.delimiter + value + this.delimiter;
        }
        return value;
    },

    /**
     * Convert the Ext.grid.View content into a list of strings.
     *
     * @return {String[]} One string per line.
     *
     */
    toCSV: function () {

        "use strict";

        var me = this,
            cells,
            columnHeader,
            csv = [],
            labels = [],
            div,
            i,
            j,
            tdEls,
            trEl,
            trEls,
            row,
            total;

        // build the row containing the column labels
        columnHeader = this.getColumnHeader();
        if (!columnHeader.hidden) {
            for (i = 0; i < columnHeader.labels.length; i += 1) {
                labels.push(this.delimit(columnHeader.labels[i]));
            }
            csv.push(labels.join(this.separator));
        }

        //
        // convert the HTML row associated to the Ext.grid.View
        // The structure of the HTML table is complex because it contains
        // several types of row depending on the grouping features.
        //
        trEls = me.getView().getEl().select("table tr");

        // process the trEls Ext.dom.CompositeElement
        for (i = 0; i < trEls.getCount(); i += 1) {

            trEl = trEls.item(i);

            // grouping row
            if (trEl.hasCls("x-grid-wrap-row")) {

                div = trEl.down(".x-grid-group-title");
                if (div) {
                    csv.push(this.delimit(div.dom.textContent));
                }

            // row with standard data cells. It includes summary row.
            } else if (trEl.hasCls("x-grid-data-row") || trEl.hasCls("x-grid-row-summary")) {

                cells = [];
                tdEls = trEl.select("td");

                // process the tdEls Ext.dom.CompositeElement
                for (j = 0; j < tdEls.getCount(); j += 1) {
                    cells.push(this.delimit(tdEls.item(j).dom.textContent));
                }

                row = cells.join(this.separator);

                // unfortunately the grand total appear as the first row (Ext JS 4.2)
                if (i === 0 && trEl.hasCls("x-grid-row-summary")) {
                    total = row;
                } else {
                    csv.push(row);
                }
            }
        }

        // push the grand total at the end
        if (total) {
            csv.push(total);
        }

        return csv;
    }
});