/**
 * This class is used as a mixin.
 * The main method translate the Ext.grid.View content into a latex string.
 *
 * @since 0.6.2
 *
 */
Ext.define('App.grid.mixin.ExportLatex', {

    extend: 'App.grid.mixin.ExportBase',

    //
    // Convert the dom Element corresponding to a grid cell into a string.
    // When the cell contains an hyper link, it is encoded as using the
    // latex command href.
    //
    // @param {Ext.dom.Element} domEl
    //
    // @return {String}
    //
    domToLaTeX: function(domEl) {

        "use strict";
        var htmlEl = domEl.dom,
            href,
            li,
            value;

        // is the cell contains an hyper link
        li = htmlEl.getElementsByTagName("A");

        // standard string
        // shrink empty string to avoid latex processing failure
        if (li.length === 0) {
            value = htmlEl.textContent.trim();

        // render the hyper link using latex command href
        } else {
            href = li.item(0).getAttribute("href");
            value = htmlEl.textContent;
            value = "\\href{" + href +"}{" + value +"}";
        }

        return value;
    },

    /**
     * Convert the Ext.grid.View content into a string.
     * The information are encoded using LaTeX standard.
     *
     * @return {String[]} One string per line.
     *
     */
    toLaTeX: function () {

        "use strict";

        var me = this,
            cells,
            cmd,
            columnsAlignment = '',
            columnHeader,
            div,
            i,
            j,
            latex = [],
            nColumns,
            tdEl,
            tdEls,
            trEl,
            trEls,
            row,
            total;

        // build the column alignment, e.g. llrrc
        columnHeader = this.getColumnHeader();
        columnHeader.alignments.forEach(function (value) {
            columnsAlignment += value[0];
        });

        // alias
        nColumns = columnHeader.alignments.length;

        // begin table
        latex.push('\\begin{longtable}{' + columnsAlignment + '}');

        // table header
        if (!columnHeader.hidden) {
            latex.push(columnHeader.labels.join(' & ') + ' \\\\');
            latex.push('\\hline\\hline');
        }

        // table content
        //
        // convert the HTML row associated to the Ext.grid.View.
        // The structure of the HTML table is complex because it contains
        // several type of row depending on the grouping features.
        //
        trEls = me.getView().getEl().select("table tr");

        // process the trEls Ext.dom.CompositeElement
        for (i = 0; i < trEls.getCount(); i += 1) {

            trEl = trEls.item(i);

            // grouping row. It is represent as a latex multicolumn
            if (trEl.hasCls("x-grid-wrap-row")) {

                div = trEl.down(".x-grid-group-title");
                if (div) {
                    cmd = '\\multicolumn{' + nColumns + '}{l}{' + me.domToLaTeX(div) + '} \\\\';
                    latex.push(cmd);
                    latex.push('\\hline');
                }

            // row with standard data cells. It includes summary row.
            } else if (trEl.hasCls("x-grid-data-row") || trEl.hasCls("x-grid-row-summary")) {

                cells = [];
                tdEls = trEl.select("td");

                // process the tdEls Ext.dom.CompositeElement
                for (j = 0; j < tdEls.getCount(); j += 1) {
                    cells.push(me.domToLaTeX(tdEls.item(j)));
                }

                row = cells.join(' & ') + ' \\\\';

                // unfortunately the grand total appear as the first row (Ext JS 4.2)
                if (i === 0 && trEl.hasCls("x-grid-row-summary")) {
                    total = row;
                } else {
                    latex.push(row);
                }
            }
        }

        // push the grand total at the end
        if (total) {
            latex.push('\\hline\\hline');
            latex.push(total);
        }

        //end table
        latex.push('\\end{longtable}');

        return latex;
    }
});