/**
 * The plugin export the content of the grid, as rendered on the screen,
 * into a file. Several formats are available: CSV, LaTeX and PDF.
 *
 * A context menu gives access to the different possibilities.
 *
 * The grid should be instantiate with the mixin App.grid.mixin.ExportToLatex.
 *
 * @since 0.6.1.8
 *
 */
Ext.define('App.grid.plugin.Export', {

    extend: 'Ext.AbstractPlugin',
    alias: 'plugin.pGridExport',
    requires: 'Ext.form.action.StandardSubmit',

    /**
     * @cfg {String}
     * The new line separator used in the CSV / LaTeX file.
     */
    newLine: '\n',

    /**
     * @cfg {Array}
     * The LaTeX preamble.
     * See for example http://fr.wikibooks.org/wiki/LaTeX/Mise_en_page.
     */
    preamble: [
        '\\documentclass{article}',
        '\\usepackage[utf8]{inputenc}',
        '\\usepackage[T1]{fontenc}',
        '\\usepackage{hyperref}',
        '\\usepackage{longtable}',
        '\\usepackage[landscape,a4paper]{geometry}',
        '\\usepackage{fancyhdr}',
        '\\geometry{tmargin=2cm,bmargin=2cm,lmargin=1.85cm,rmargin=1.85cm}',
        '\\pagestyle{fancy}',
        '\\fancyhead{}',
        '\\fancyfoot{}',
        '\\rfoot{\\thepage}'
    ],

    // private properties for internationalization
    textToCSV: 'Export to CSV...',
    textToLaTeX: 'Export to LaTeX...',
    textToPDF: 'Export to PDF...',

    // private short cut
    menu: null,

    // Pre-defined configuration
    pluginId: 'gridExport',

    /**
     * Initialize the plugin
     *
     * @param {App.grid.Panel} grid
     *
     */
    init: function (grid) {

        "use strict";

        // keep track of the grid
        this.setCmp(grid);

        // the context menu
        this.menu = Ext.create('Ext.menu.Menu', {
            items : [{
                text: this.textToCSV,
                iconCls: 'xminetype-csv',
                handler: this.onCsvExport,
                scope: this
            }, {
                text: this.textToLaTeX,
                iconCls: 'xminetype-tex',
                handler: this.onLatexExport,
                scope: this
            }, {
                text: this.textToPDF,
                iconCls: 'xminetype-pdf',
                handler: this.onPdfExport,
                scope: this
            }]
        });

        // the menu pops up everywhere in the the grid, no context menu outside.
        // Move from the Ext.component to the Ext.dom.element space.
        grid.on('render', function(cmp){

            cmp.body.on("contextmenu", function(event){
                event.stopEvent();
                this.menu.showAt(event.getXY());
            }, this);

            cmp.body.up(".x-panel-body").on("contextmenu", function(event){
                event.stopEvent();
            }, this);

        }, this, {single: true});
    },

    /**
     * Destroy the plugin
     *
     */
    destroy: function () {

        "use strict";
        Ext.destroy(this.memu);
    },

    //
    // Do latex
    // @return {String[]} one string per line
    //
    doLaTeX: function () {

        "use strict";

        var latex = [];

        // latex preamble
        latex = latex.concat(this.preamble);

        // begin document
        latex.push('\\begin{document}');

        // the table
        latex = latex.concat(this.getCmp().toLaTeX());

        // end document
        latex.push('\\end{document}');

        return latex;
    },

    /**
     * Export the content of the grid as a CSV file.
     *
     */
    onCsvExport: function () {

        "use strict";

        var csv = this.getCmp().toCSV();
        App.save("text/csv", "utf-8", csv);
    },

    /**
     * Export the content of the grid as a LaTex file.
     *
     */
    onLatexExport: function () {

        "use strict";

        var latex = this.doLaTeX();
        App.save("application/x-latex", "utf-8", latex);
    },

    /**
     * Export the content of the grid as a PDF file.
     * The latex file is generated on the client side and transfer to
     * the server where the conversion take place.
     *
     */
    onPdfExport: function () {

       "use strict";

        var data,
            form;

        data = this.doLaTeX().join(this.newLine);

        // send the latex data to the server.
        // the method used standardSubmit using a form.
        form = Ext.create('Ext.form.Panel', {
            items: [{
                name: 'latex',
                value: data,
                xtype:'textfield'
            }],
            standardSubmit: true,
            url: App.latex2pdfUrl
        });

        form.getForm().submit();
    }
});