/**
 * The plugin instantiating the row editor.
 * The widget is a Ext.Window containing the App.form.Panel.
 * It exposes series of handlers to delete, duplicate, update, view the
 * selected row of the App.grid.Panel, as well as to create new row.
 *
 * **Note**: This plugin limits the number of selected row to one.
 *
 */
Ext.define('App.grid.plugin.RowEditorBase',  {

    extend: 'Ext.AbstractPlugin',
    alias: 'plugin.pGridRowEditor',
    requires: [
        'Ext.layout.container.Fit',
        'Ext.window.Window'
    ],
    uses: 'App.form.Panel',

    /**
     * @cfg {Array} [resetFields="TableId"]
     * A list of fields which are reset to their defaults
     * during the duplicate operation. the field are identified by
     * their name defined in the Ext.data.Field.
     * A typical example is the reset of a status from OK to undefined.
     */
    resetFields: null,

    // Pre-defined configuration
    pluginId: 'rowEditor',

    // Private short cuts
    formPanel: null,
    window: null,

    // Private attributes for internationalization
    addTitle: "Create a new record...",
    deleteTitle: "Delete the record...",
    duplicateTitle: "Duplicate the record...",
    editTitle: "Update the record...",
    viewTitle: "View the record...",
    textMsg: 'Select a row please',

    /**
     * Initialize the plugin
     *
     * @param {App.grid.Panel} grid
     */
    init: function (grid) {

        "use strict";

        var table,
            store = grid.getStore();

        this.setCmp(grid);

        // The user can only select one row
        grid.getSelectionModel().setSelectionMode('SINGLE');

        // Add a listener to hide the window when the transaction is successful
        // or when the transaction failed
        store.on('write', this.onWrite, this);
        store.getProxy().on('exception', this.onException, this);

        // instantiate the window
        this.window = Ext.create('Ext.window.Window', {
            closeAction: 'hide',
            layout: 'fit',
            modal: true,
            plain: true
        });

        // get the formPanel configuration and instantiate the object
        table = App.getTableName(grid.getStore());
        Dbui.getForm(table, this.addFormPanel, this);

        // initialise the list of fields to be reset on duplicate
        // the field id is mandatory
        if (!this.pluginConfig.resetFields) {
            this.resetFields = [];
        }
        this.resetFields.push(App.encodeField(table, 'id'));
    },

    /**
     * Instantiate the App.form.Panel.
     * The scope is the RowEditor.
     *
     * @param {Ext.direct.Provider} provider
     *
     * @param {Object} response
     * @param {String} response.type
     * The type of the transaction: rpc
     *
     * @param {Number} response.tid
     * The transaction id that had just been processed.
     *
     * @param {Object} response.action
     * The class action that has just been processed.
     *
     * @param {String} response.method
     * The method that has just been processed.
     *
     * @param {Object} response.result
     * The configuration of the App.form.Panel.
     *
     */
    addFormPanel: function (provider, response) {

        "use strict";

        var formCfg = response.result;

        // instantiate the form
        this.formPanel = Ext.create('App.form.Panel', formCfg);
        this.window.add(this.formPanel);
    },

    /**
     * Destroy the plugin
     *
     */
    destroy: function () {

        "use strict";

        var store;

        store = this.getCmp().getStore();
        store.un('write', this.onWrite, this);
        store.getProxy().un('exception', this.onException, this);

        Ext.destroyMembers(this, 'formPanel');
        Ext.destroyMembers(this, 'window');
    },

    /**
     * Return the record associated to the row selected in the grid.
     *
     * @return {Ext.data.Record/Boolean}
     * In absence of selected row a warning message is shown
     * and the function return false.
     *
     */
    getSelected: function () {

        "use strict";

        var record = this.getCmp().getSelectionModel().getLastSelected();

        if (!record) {
            Ext.MessageBox.alert('Warning', this.textMsg);
            return false;
        }

        return record;
    },

    /**
     * Add a new row.
     */
    onAddRow: function () {

        "use strict";

        this.setCurrentIndex();

        this.formPanel.setAction('create');
        this.window.setTitle(this.addTitle);
        this.window.show();
    },

    /**
     * Delete the selected row without confirmation.
     */
    onDeleteRow: function () {

        "use strict";

        var record = this.getSelected();
        if (!record) {return; }

        this.formPanel.setAction('destroy', record);
        this.window.setTitle(this.deleteTitle);
        this.formPanel.doAction();
    },

    /**
     * Duplicate the selected row.
     */
    onDuplicateRow: function () {

        "use strict";

        var i,
            record = this.getSelected(),
            value;

        if (!record) {return; }

        this.setCurrentIndex();

        // duplicate the record and reset at least the field id
        //
        // NOTE: the method record.set is not used since it change
        // the record as dirty which trigger update on the server
        //
        record = record.copy();

        for (i = 0; i < this.resetFields.length; i += 1) {
            value = record.fields.get(this.resetFields[i]).defaultValue;
            record.data[this.resetFields[i]] = value;
        }

        this.formPanel.setAction('duplicate', record);
        this.window.setTitle(this.duplicateTitle);
        this.window.show();
    },

    /**
     * Edit the selected row.
     */
    onEditRow: function () {

        "use strict";

        var record = this.getSelected();
        if (!record) {return; }

        this.formPanel.setAction('update', record);
        this.window.setTitle(this.editTitle);
        this.window.show();
    },

    /**
     * The transaction is reject by the server due to an invalid field
     * or most probably by a callback procedure.
     *
     * Hide the form when the transaction failed.
     * Restore the content of the grid to the values stored on the server.
     *
     * NOTE (0.6.1.10):
     * a more elaborated policy might be required, e.g. only restore
     * database values when a callback fires the error. To be seen.
     *
     */
    onException: function () {

        "use strict";

        var store = this.getCmp().getStore();

        this.window.hide();
        store.loadPage(store.currentPage);
    },

    /**
     * View the selected row in the form.
     */
    onViewRow: function () {

        "use strict";

        var record = this.getSelected();
        if (!record) {return; }

        this.formPanel.setAction('view', record);
        this.window.setTitle(this.viewTitle);
        this.window.show();
    },

    /**
     * Fire when the write transaction is successful.
     * Its main action is to hide the window.
     *
     * @param {Ext.data.Store} store
     *
     * @param {String} action
     *
     * @param {Object} result
     * The 'data' picked-out out of the response for convenience.
     *
     * @param {Object} transaction
     *
     * @param {Ext.data.Record} record
     */
    onWrite: function (store, action, result, transaction, record) {
        "use strict";
        this.window.hide();
    },

    /**
     * Get the index of the selected record and fill
     * the property #formPanel.indexSelectedRecord.
     */
    setCurrentIndex: function () {

        "use strict";

        var index,
            record;

        // reset
        this.formPanel.indexSelectedRecord = undefined;

        // get the selected record
        record = this.getCmp().getSelectionModel().getLastSelected();

        // update
        if (record) {
            index = this.getCmp().store.indexOf(record);
            this.formPanel.currentIndex = index;
        }
    }
});