/**
 * The Panel with URL selector is split in two parts the central panel
 * and the selector.
 * The selector display a list of {@link Ext.form.Field fields} organized in
 * {@link Ext.form.FieldSet field set}.
 * The URL content is displayed in the main panel.
 *
 * The URL display in the main panel is defined by the #baseUrl option.
 * It can be modified dynamically by extracting the name of the
 * *controller*, *function* and/or *extension* from the selector fields.
 * The options #ctrlField, #extField and #funcField determine
 * the field mappings.
 *
 */
Ext.define('App.panel.WithUrlSelector', {

    extend: 'App.panel.BaseWithSelector',
    alias: 'widget.xpanelwithurlselector',

    /**
     * @cfg {Object} baseParams
     * Params that will be attached to every URL request.
     * These parameters will not be overridden by the load options.
     */
    baseParams: null,

    /**
     * @cfg {String}
     * The well-formed URL, e.g http://blabla.
     */
    baseUrl: undefined,

    /**
     * @cfg {String}
     * The name of the field defining the controller.
     * The URL becomes **baseUrl/ctrlFieldValue**.
     */
    ctrlField: undefined,

    /**
     * @cfg {String}
     * The name of the field defining the extension.
     * The URL becomes **baseUrl.extFieldValue**.
     */
    extField: undefined,

    /**
     * @cfg {String}
     * The name of the field defining the function.
     * The URL becomes **baseUrl/funcFieldValue**.
     */
    funcField: undefined,

    /**
     * @cfg {Number}
     * The time out for the {@link Ext.Ajax Ajax request} in millisecond.
     */
    timeout: 30000,

    // Private properties for internationalization
    textLoad: 'Loading...',
    textProgressBar: 'Preparing your file, please wait...',

    // Private method requires by the ExtJS model
    initComponent: function () {

        this.callParent(arguments);

        if (this.baseParams) {
            this.mainPanel.loader.baseParams = this.baseParams;
        }

        this.goButton.on('click', this.onGo, this);
        this.resetButton.on('click', this.onReset, this);
    },

    // Private method requires by the Ext JS component model
    beforeDestroy: function () {

        this.goButton.un('click', this.onGo, this);
        this.resetButton.un('click', this.onReset, this);

        this.callParent(arguments);
    },

    // Private method to extract the URL parameters
    // @return {Object} the URL parameters
    //
    // @return {String}
    // return.ctrl the name of the requested controller
    //
    // @return {String}
    // return.func the name of the requested function
    //
    // @return {String} return.ext
    // the name of the requested extension
    //
    // @return {Object} return.vars
    // object containing the parameters passed into the body of the request
    //
    // @return {String} return.http
    //
    getUrlParams: function () {

        "use strict";

        var i,
            fields,
            fieldName,
            fieldValue,
            li,
            fName,
            url =  {vars: {}};

        function extract(key, subfield) {
            url.vars[subfield.getName()] = subfield.getValue();
        }

        fields = this.selectorPanel.query('field');
        for (i = 0; i < fields.length; i += 1) {

            fieldName = fields[i].getName();
            fieldValue = fields[i].getValue();

            // the field name is encode TableField
            li = App.decodeCapitalizeWords(fieldName);
            fName = li[1];

            switch (fName) {

            case this.ctrlField:
                url.ctrl = fieldValue;
                break;

            case this.extField:
                url.ext = fieldValue;
                break;

            case this.funcField:
                url.func = fieldValue;
                break;

            default:
                url.vars[fieldName] = fieldValue;
                break;
            }
        }

        // complete URL parameters with those embedded in composite fields
        fields = this.selectorPanel.query('fieldcontainer');
        for (i = 0; i < fields.length; i += 1) {
            fields[i].items.eachKey(extract);
        }

        // build the URL
        url.http = this.baseUrl;

        if (url.ctrl !== undefined) {
            url.http = url.http + '/' + url.ctrl;
        }

        if (url.func !== undefined) {
            url.http = url.http + '/' + url.func;
        }

        if (url.ext !== undefined) {
            url.http = url.http + '.' + url.ext;
        }

        return url;
    },

    /**
     * Handler to build the URL and to load it in the panel.
     * The main panel is masked and the selector panel is disabled
     * during the loading.
     */
    onGo: function () {

        "use strict";

        var iframe,
            mask,
            request,
            url,
            vars;

        // destroy embedded IFRAME in the mainPanel if any
        iframe = this.mainPanel.getEl().getById('mainPanelIframe');
        if (iframe) {
            Ext.destroy(iframe);
        }

        // activate the auto scrolling
        this.mainPanel.setAutoScroll(true);

        // extract URL parameters
        request = this.getUrlParams();

        // load the HTML content in the main panel
        // use the main panel loader
        // mask the whole viewport
        if (!request.ext || request.ext.indexOf('htm') !== -1) {

            mask = {
                msg: this.textLoad,
                target: this.up('xviewport')
            };

            this.mainPanel.getLoader().load({
                loadMask: mask,
                params: request.vars,
                url: request.http,
                timeout: this.timeout
            });

            return;
        }

        // Download the CSV, PDF, ... file using IFRAME
        // launch a progress bar
        Ext.MessageBox.show({
            msg: this.textProgressBar,
            progressText: 'Saving...',
            width: 300,
            wait: true
        });

        // build the URL using the GET method
        vars = Ext.Object.merge(request.vars, this.baseParams);
        url = request.http + '?' + Ext.Object.toQueryString(vars);

        // append the IFRAME to the main panel therefore
        // the browser will launch the downloading.
        //
        // NOTE: the IFRAME cover almost the full space.
        // this is useful when the PDF is embedded in the panel.
        //
        // NOTE: keep the background colour to white
        //
        iframe = this.mainPanel.getEl().createChild({
            tag: 'iframe',
            id: 'mainPanelIframe',
            frameborder: 0,
            width: '100%',
            height: '99%',
            style: 'background: #FFFFFF;',
            src: url
        });

        // when the download is finish hide message box
        iframe.on('load', function () {
            Ext.MessageBox.hide();
        }, this, {single: true});
    },

    /**
     * Handler to reset the selector
     */
    onReset: function () {

        "use strict";

        var fields,
            i;

        fields = this.selectorPanel.query('field');
        for (i = 0; i < fields.length; i += 1) {
            fields[i].reset();
        }
    }
});